@@ -13,8 +13,10 @@
 # under the License.
 
 import collections
+import inspect
 import json
 import mock
+import tenacity
 
 from paunch.builder import compose1
 from paunch import runner
@@ -24,6 +26,8 @@ from paunch.tests import base
 class TestComposeV1Builder(base.TestCase):
 
     def test_apply(self):
+        orig_call = tenacity.wait.wait_random_exponential.__call__
+        orig_argspec = inspect.getargspec(orig_call)
         config = {
             'one': {
                 'start_order': 0,
@@ -53,7 +57,8 @@ class TestComposeV1Builder(base.TestCase):
         exe.side_effect = [
             ('exists', '', 0),  # inspect for image centos:6
             ('', '', 1),  # inspect for missing image centos:7
-            ('Pulled centos:7', '', 0),  # pull centos:6
+            ('Pulled centos:7', 'ouch', 1),  # pull centos:6 fails
+            ('Pulled centos:7', '', 0),  # pull centos:6 succeeds
             ('', '', 0),  # ps for delete_missing_and_updated container_names
             ('', '', 0),  # ps for after delete_missing_and_updated renames
             ('', '', 0),  # ps to only create containers which don't exist
@@ -67,7 +72,12 @@ class TestComposeV1Builder(base.TestCase):
         r.unique_container_name = lambda n: '%s-12345678' % n
         r.execute = exe
 
-        builder = compose1.ComposeV1Builder('foo', config, r)
+        with mock.patch('tenacity.wait.wait_random_exponential.__call__') as f:
+            f.return_value = 0
+            with mock.patch('inspect.getargspec') as mock_args:
+                mock_args.return_value = orig_argspec
+                builder = compose1.ComposeV1Builder('foo', config, r)
+
         stdout, stderr, deploy_status_code = builder.apply()
         self.assertEqual(0, deploy_status_code)
         self.assertEqual([
@@ -91,6 +101,11 @@ class TestComposeV1Builder(base.TestCase):
                 ['docker', 'inspect', '--type', 'image',
                  '--format', 'exists', 'centos:7']
             ),
+            # first pull attempt fails
+            mock.call(
+                ['docker', 'pull', 'centos:7']
+            ),
+            # second pull attempt succeeds
             mock.call(
                 ['docker', 'pull', 'centos:7']
             ),
@@ -302,6 +317,8 @@ three-12345678 three''', '', 0),
         ])
 
     def test_apply_failed_pull(self):
+        orig_call = tenacity.wait.wait_random_exponential.__call__
+        orig_argspec = inspect.getargspec(orig_call)
         config = {
             'one': {
                 'start_order': 0,
@@ -332,10 +349,18 @@ three-12345678 three''', '', 0),
             ('exists', '', 0),  # inspect for image centos:6
             ('', '', 1),  # inspect for missing image centos:7
             ('Pulling centos:7', 'ouch', 1),  # pull centos:7 failure
+            ('Pulling centos:7', 'ouch', 1),  # pull centos:7 retry 2
+            ('Pulling centos:7', 'ouch', 1),  # pull centos:7 retry 3
+            ('Pulling centos:7', 'ouch', 1),  # pull centos:7 retry 4
         ]
         r.execute = exe
 
-        builder = compose1.ComposeV1Builder('foo', config, r)
+        with mock.patch('tenacity.wait.wait_random_exponential.__call__') as f:
+            f.return_value = 0
+            with mock.patch('inspect.getargspec') as mock_args:
+                mock_args.return_value = orig_argspec
+                builder = compose1.ComposeV1Builder('foo', config, r)
+
         stdout, stderr, deploy_status_code = builder.apply()
         self.assertEqual(1, deploy_status_code)
         self.assertEqual(['Pulling centos:7'], stdout)
@@ -434,6 +459,7 @@ three-12345678 three''', '', 0),
                 'interactive': True,
                 'environment': ['FOO=BAR', 'BAR=BAZ'],
                 'env_file': ['/tmp/foo.env', '/tmp/bar.env'],
+                'ulimit': ['nofile=1024', 'nproc=1024'],
                 'volumes': ['/foo:/foo:rw', '/bar:/bar:ro'],
                 'volumes_from': ['two', 'three']
             }
@@ -447,6 +473,7 @@ three-12345678 three''', '', 0),
              '--env-file=/tmp/foo.env', '--env-file=/tmp/bar.env',
              '--env=FOO=BAR', '--env=BAR=BAZ',
              '--rm', '--interactive', '--tty',
+             '--ulimit=nofile=1024', '--ulimit=nproc=1024',
              '--volume=/foo:/foo:rw', '--volume=/bar:/bar:ro',
              '--volumes-from=two', '--volumes-from=three',
              'centos:7', 'ls', '-l', '/foo'],
